<?php

namespace App\Console\Commands;

use App\Models\BpocWebsiteList;
use Illuminate\Console\Command;

/**
 * BPOC site list importer
 *
 * Command to import BPOC details about links
 *
 * @author Lucas Buzzo <lucas.buzzo@ab-inbev.com>
 * @package App\Console\Commands
 */

class BpocSiteListImporter extends Command
{
    /**
     * The console command description.
     */
    protected $description = 'Import BPOC informations to generate website list';

    /**
     * The name and signature of the console command.
     */
    protected $signature = 'import:bpoc
    {filename : CSV file}
    {linkColumn : which column is the link}
    {typeColumn : which column is asset type}
    {statusColumn : which column is the status}
    {zoneColumn : which column is the zone}
    {countryColumn : which column is the country}
    {brandColumn : which column is the brand}
    {dgpScopeColumn : which column is the dgpScope}';

    /**
     * Convert Excel column to array index
     *
     * @param string $column
     *
     * @return int
     */
    protected static function column2index(string $column): int
    {
        $number = 0;
        foreach (str_split($column) as $letter) {
            $number = ($number * 26) + (ord(strtolower($letter)) - 96);
        }
        return $number - 1;
    }

    /**
     * Execute the console command.
     *
     * @return int
     */
    public function handle(): int
    {
        $filename = $this->argument('filename');

        $linkColumn = strtoupper($this->argument('linkColumn'));
        $typeColumn = strtoupper($this->argument('typeColumn'));
        $statusColumn = strtoupper($this->argument('statusColumn'));
        $zoneColumn = strtoupper($this->argument('zoneColumn'));
        $countryColumn = strtoupper($this->argument('countryColumn'));
        $dgpScopeColumn = strtoupper($this->argument('dgpScopeColumn'));
        $brandColumn = strtoupper($this->argument('brandColumn'));

        $linkColumn = $this->column2index($linkColumn);
        $typeColumn = $this->column2index($typeColumn);
        $statusColumn = $this->column2index($statusColumn);
        $zoneColumn = $this->column2index($zoneColumn);
        $countryColumn = $this->column2index($countryColumn);
        $dgpScopeColumn = $this->column2index($dgpScopeColumn);
        $brandColumn = $this->column2index($brandColumn);

        $file = fopen($filename, 'r');

        BpocWebsiteList::truncate();

        while (($line = fgetcsv($file)) !== FALSE) {
            if (trim(strtolower($line[$typeColumn])) !== 'website') {
                continue;
            }

            if (!$this->isValidUrl($line[$linkColumn])) {
                continue;
            }

            $entry = new BpocWebsiteList();

            $entry->link = strtolower($line[$linkColumn]);
            $entry->status = strtolower($line[$statusColumn]);

            if (!empty($line[$zoneColumn])) {
                $entry->zone = utf8_encode($line[$zoneColumn]);
            }

            if (!empty($line[$countryColumn])) {
                $entry->country = utf8_encode($line[$countryColumn]);
            }

            if (!empty($line[$brandColumn])) {
                $entry->brand = utf8_encode($line[$brandColumn]);
            }

            $entry->dgp_scope = (trim(strtolower($line[$dgpScopeColumn])) === 'yes' ? true : false);

            $entry->save();
        }

        fclose($file);

        return 0;
    }

    /**
     * Validate if url given is valid
     *
     * @param string $link
     *
     * @return boolean
     */
    protected function isValidUrl(string $link): bool
    {
        $valid = false;

        if (filter_var($link, FILTER_VALIDATE_URL) === FALSE) {
            $valid = true;
        }

        return $valid;
    }
}
