<?php
// phpcs:disable BestIt.DocTags.RequiredPropertyTags.TagOccurrenceMinVar
// phpcs:disable BestIt.TypeHints.PropertyTypeHint.MissingAnyTypeHint

namespace App\Console\Commands;

use App\Console\CrawlerHelper;
use App\Models\Links;
use Illuminate\Console\Command;

/**
 * CheckCriteria Class
 *
 * Command to select which validations would run
 *
 * @author Lucas Buzzo <lucas.buzzo@ab-inbev.com>
 * @package App\Console\Commands
 */
class CheckCriteria extends Command
{
	/**
	 * The signature to call the command
	 */
	protected $signature = 'crawler:start
	{official? : Define if is an official execution}
	{channel? : If need to evaluate only one specific channel}
	{zone? : If need to evaluate only one specific zone}
	{country? : If need to evaluate only one specific country}
	{brand? : If need to evaluate only one specific brand}
	{executed_at? : Run with a specific execution datetime}
	{--cookies : Also run cookie crawler}';

	/**
	 * Handle the command
	 *
	 * @return int
	 */
	public function handle(): int
	{
		$zone = $this->argument('zone');
		$country = $this->argument('country');
		$brand = $this->argument('brand');
		$channel = $this->argument('channel');
		$links = Links::with('channel');
		$official = ($this->argument('official') ? $this->argument('official') : false);
		$executedParam = $this->argument('executed_at');

		$links->join('channels', 'channels.id', '=', 'links.channel_id');
		$links->join('brands', 'brands.id', '=', 'links.brand_id');
		$links->join('countries', 'countries.id', '=', 'links.country_id');
		$links->join('locales', 'locales.id', '=', 'countries.locale_id');
		$links->join('zones', 'zones.id', '=', 'countries.zone_id');
		$links->orderBy('links.link');

		if ($channel) {
			$links->where('channels.slug', $channel);
		}

		if ($brand) {
			$links->where('brands.name', $brand);
		}

		if ($country) {
			$links->where('countries.name', $country);
		}

		if ($zone) {
			$links->where('zones.name', $zone);
		}

		$executedAt = null;
		if ($executedParam) {
			$executedAt = $executedParam;
		} else {
			$executedAt = date('Y-m-d H:i:s');
		}

		// We'll only check consider sites
		$links->where('links.consider', true);

		$this->line('Starting ' . date('m/d/Y h:i:s a', time()));

		$crawled = [];

		app(CrawlerHelper::class)->sendNotification('CheckCriteria: Starting Crawlers');

		// $run = array('');
		$linksDb = $links->get([
			'links.link as link',
			'locales.locale as locale',
			'channels.slug as channelSlug',
			'countries.name as countryName',
			'zones.name as zone',
			'links.alcoholic as alcoholic',
			'links.additional_crawler as additional_crawler',
			'links.in_acquia as in_acquia',
		]);
		foreach ($linksDb as $link) {
			if (!(in_array($link->link, $crawled))) {
				// if (!(in_array($link->link, $run))) continue;

				$ignoreInCriteria = false;
				// @phpstan-ignore-next-line
				if ($link->countryName === 'Ukraine' || $link->countryName === 'Russia') $ignoreInCriteria = true;

				$this->line('Starting ... ' . $link->link);

				if (!$link->locale) { // @phpstan-ignore-line
					$this->error('No locale for country ' . $link->countryName);
					continue (1);
				}

				// @phpstan-ignore-next-line
				switch ($link->channelSlug) {
					case 'twitter':
						$this->call('crawler:twitter', [
							'link' => $link->link,
							'locale' => $link->locale,
							'executed_at' => $executedAt,
							'official' => $official,
						]);
						break;
					case 'site':
						$this->call('crawler:site', [
							'link' => $link->link,
							'locale' => $link->locale,
							'alcoholic' => $link->alcoholic,
							'additionalCrawler' => $link->additional_crawler,
							'executed_at' => $executedAt,
							'official' => $official,
							'ignoreInCriteria' => $ignoreInCriteria,
						]);
						break;
					case 'youtube':
						$this->call('crawler:youtube', [
							'link' => $link->link,
							'locale' => $link->locale,
							'executed_at' => $executedAt,
							'official' => $official,
						]);
						break;
					case 'instagram':
						$this->call('crawler:instagram', [
							'link' => $link->link,
							'locale' => $link->locale,
							'executed_at' => $executedAt,
							'official' => $official,
						]);
						break;
					case 'facebook':
						$this->call('crawler:facebook', [
							'link' => $link->link,
							'locale' => $link->locale,
							'executed_at' => $executedAt,
							'official' => $official,
						]);
						break;
					default:
						break;
				}

				if ($official) {
					app(CrawlerHelper::class)->storeManualValidationHelper(
						link: $link->link,
						exec_date: $executedAt,
						validated: false,
						use_on_report: true,
					);
				}
			}

			array_push($crawled, $link->link);
		}

		if ($this->option('cookies')) {
			$this->line('Updating Onetrust Informations for cookies');

			$this->call('cookie:criteria', [
				'--update-ot-table' => 1,
			]);

			$this->line('Updated Onetrust informations for cookies');

			$this->call('cookie:criteria', [
				'--crawl-bpoc' => $executedAt,
				'--official' => $official,
			]);
		}

		if ($channel === 'site' || empty($channel)) {
			$this->call('ga:criteria', [
				'executed_at' => $executedAt,
				'official' => $official,
			]);

			$this->call('crawler:semrush', [
				'executed_at' => $executedAt,
				'official' => $official,
			]);
		}

		if ($channel === 'twitter' || empty($channel)) {
			$this->call('twitter:unfollow');
		}

		// We'll only update the MV if there are a official execution
		if ($official) {
			$this->call('refresh:report_draft');
		}

		$this->line('Finished ' . date('m/d/Y h:i:s a', time()));

		app(CrawlerHelper::class)->sendNotification('CheckCriteria: Finished!');

		return Command::SUCCESS;
	}
}
