<?php

// phpcs:disable BestIt.DocTags.RequiredPropertyTags.TagOccurrenceMinVar
// phpcs:disable BestIt.TypeHints.PropertyTypeHint.MissingAnyTypeHint
// phpcs:disable PSR2.ControlStructures.ElseIfDeclaration.NotAllowed

namespace App\Console\Commands;

use App\Console\CrawlerHelper;
use App\Models\BpocWebsiteList;
use App\Models\CookieComplianceReport;
use App\Models\CookiesWebsite;
use App\Models\Links;
use App\Models\OnetrustCookies;
use Exception;
use Illuminate\Console\Command;
use Illuminate\Support\Facades\Artisan;
use stdClass;

ini_set('memory_limit', '24095M');
ini_set('max_execution_time', '0');
set_time_limit(0);

/**
 * Cookie Criteria Checker
 *
 * Command to check the cookie compliance based on Onetrust data
 *
 * @author Lucas Buzzo <lucas.buzzo@ab-inbev.com>
 * @package App\Console\Commands
 */
class CookieCriteria extends Command
{
    /**
     * Hold the access token
     *
     * @var string
     */
    protected string $accessToken = '';

    /**
     * The console command description.
     */
    protected $description = 'Command to check the cookie compliance based on Onetrust data';

    /**
     * The name and signature of the console command.
     */
    protected $signature = 'cookie:criteria
        {--dump-websites : Dump website informations}
        {--dump-cookies : Dump cookie informations for all sites inside OT}
        {--report-cookies : Generate report (csv) at cookie level with the lastest information}
        {--report-domain=? : Generate report (csv) at domain level regarding compliance}
        {--update-ot-table : Update the OnetrustCookies table}
        {--update-ot-cookies : Update only Onetrust Cookies data}
        {--crawl-bpoc=? : Run the crawler at BPOC site list, with specified executed at}
        {--official=? : If this execution is official}
        {--update-script-status : Update OT script status for the domains}
        ';

    /**
     * For a given link, retrieve the additional crawler
     *
     * @param string $link
     *
     * @return string
     */
    protected function getAdditionalCrawler(string $link): string
    {
        $additional = '';

        try {
            $data = Links::where('link', 'ilike', $link)->first();

            if (!is_null($data)) {
                $additional = $data->additional_crawler;
            } else {
                $additional = '{}';
            }
        } catch (Exception $e) {
            $additional = '{}';
            $this->error('Failed to get additional crawler for ' . $link . '. Error message: ' . $e->getMessage());
        } finally {
            if (is_null($additional)) {
                $additional = '{}';
            }

            return $additional;
        }
    }

    /**
     * Get Cookies from Onetrust for the given domain
     *
     * @param string $site
     * @param integer $page
     *
     * @return array
     */
    protected function getCookies(string $site, int $page = 0): array
    {
        $cookies = [];

        try {
            $curl = curl_init();

            curl_setopt_array($curl, array(
                CURLOPT_URL => 'https://app-de.onetrust.com/api/cookiemanager/v2/cookie-reports/search?language=en&page=' . $page,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_ENCODING => '',
                CURLOPT_MAXREDIRS => 10,
                CURLOPT_TIMEOUT => 0,
                CURLOPT_FOLLOWLOCATION => true,
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => 'POST',
                CURLOPT_POSTFIELDS => '{"domains" : ["' . $site . '"]}',
                CURLOPT_HTTPHEADER => array(
                    'Authorization: Bearer ' . $this->accessToken,
                    'Content-Type: application/json'
                ),
            ));

            $response = curl_exec($curl);

            if (curl_getinfo($curl, CURLINFO_HTTP_CODE) === 401) {
                $this->initializeOnetrust();

                $cookies = array_merge($cookies, $this->getCookies($site, $page));
            }

            $decoded = json_decode($response);

            if (is_null($decoded)) {
                $decoded = new stdClass();
                $decoded->content = [];
            } else {
                if (!isset($decoded->content)) {
                    $decoded->content = [];
                }
            }

            curl_close($curl);

            $cookies = $decoded->content;

            if (isset($decoded->totalPages)) {
                if ($decoded->totalPages > ($page + 1)) {
                    $cookies = array_merge($cookies, $this->getCookies($site, $page + 1));
                }
            }
        } catch (Exception $e) {
            $this->error('Failed at ' . $e->getLine() . '. Error message: ' . $e->getMessage());
            $cookies = [];
        } finally {
            return $cookies;
        }
    }

    /**
     * Get OT Script Status
     *
     * @param string $domainId
     *
     * @return String
     */
    protected function getScriptStatus(string $domainId): String
    {
        $curl = curl_init();

        $isJson = false;

        $response = '';

        curl_setopt_array($curl, array(
            CURLOPT_URL => 'https://app-de.onetrust.com/api/cookiemanager/v1/script-integration/' . $domainId . '/downloadscriptv2',
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'GET',
            CURLOPT_HTTPHEADER => array(
                'Authorization: Bearer ' . $this->accessToken
            ),
        ));

        $response = curl_exec($curl);

        if (curl_getinfo($curl, CURLINFO_HTTP_CODE) === 401) {
            $this->initializeOnetrust();

            $response = $this->getScriptStatus($domainId);
        } else {
            $isJson = (!(is_null(json_decode($response))));

            $response = ($isJson ? 'Draft' : 'Published');
        }

        curl_close($curl);

        return $response;
    }

    /**
     * Retrive the lis of websites from Onetrust
     *
     * @return array
     */
    protected function getWebsites(int $page = 0): array
    {
        $websites = [];

        try {
            $curl = curl_init();

            $curl = curl_init();

            curl_setopt_array($curl, array(
                CURLOPT_URL => 'https://app-de.onetrust.com/api/cookiemanager/v2/websites?page=' . $page,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_ENCODING => '',
                CURLOPT_MAXREDIRS => 10,
                CURLOPT_TIMEOUT => 0,
                CURLOPT_FOLLOWLOCATION => true,
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => 'GET',
                CURLOPT_HTTPHEADER => array(
                    'Authorization: Bearer ' . $this->accessToken
                ),
            ));

            $response = curl_exec($curl);

            if (curl_getinfo($curl, CURLINFO_HTTP_CODE) === 401) {
                $this->initializeOnetrust();

                $websites = array_merge($websites, $this->getWebsites($page));
            }

            curl_close($curl);

            $response = json_decode($response);

            $websites = $response->content;

            if ($response->totalPages > ($page + 1)) {
                $websites = array_merge($websites, $this->getWebsites($page + 1));
            }
        } catch (Exception $e) {
            $this->error('Failed at ' . $e->getLine() . '. Error message: ' . $e->getMessage());
            $websites = [];
        } finally {
            return $websites;
        }
    }

    /**
     * Execute the console command.
     *
     * @return int
     */
    public function handle(): int
    {
        $return = 0;
        try {
            //TODO: we shouldn't initialize this when just crawling
            $this->initializeOnetrust();

            if ($this->option('dump-websites')) {
                $websites = $this->getWebsites();

                $header = [
                    'domainName',
                    'domainId',
                    'status',
                    'lastScannedDate',
                    'lastScannedTotalCookies',
                    'lastScannedTotalPages',
                    'scheduledDateOfNextScan',
                    'consentPolicyName',
                    'orgName',
                ];

                $table = [];

                array_push($table, $header);

                foreach ($websites as $site) {
                    array_push($table, [
                        $site->domainName,
                        $site->domainId,
                        $site->status,
                        $site->lastScannedDate,
                        $site->lastScannedTotalCookies,
                        $site->lastScannedTotalPages,
                        $site->scheduledDateOfNextScan,
                        $site->consentPolicyName,
                        $site->orgName,
                    ]);
                }

                echo $this->str_putcsv($table);
            } else if ($this->option('dump-cookies')) {
                $this->updateCookieInformation();

                $header = [
                    'domain',
                    'cookieName',
                    'lifespan',
                    'cookie_host',
                    'defaultCategory',
                    'cookieSource',
                    'thirdParty',
                    'expiry',
                ];

                $table = [];

                array_push($table, $header);

                $cookies = OnetrustCookies::all();

                foreach ($cookies as $cookie) {
                    array_push($table, [
                        $cookie->domain,
                        $cookie->cookieName,
                        $cookie->lifespan,
                        $cookie->cookie_host,
                        $cookie->defaultCategory,
                        $cookie->cookieSource,
                        $cookie->thirdParty,
                        $cookie->expiry,
                    ]);
                }

                echo ($this->str_putcsv($table));
            } else if ($this->option('update-ot-table')) {
                $this->updateCookieInformation();
            } else if ($this->option('update-ot-cookies')) {
                $this->updateCookieInformation(false, false);
            } else if ($this->option('report-cookies')) {
                $cookies = CookiesWebsite::all();

                $header = [
                    '0' => 'domain',
                    '1' => 'zone',
                    '2' => 'country',
                    '3' => 'DGP scope',
                    '4' => 'cookie',
                    '5' => 'trigger',
                    '6' => 'expected',
                    '7' => 'status',
                ];

                $table = [];
                $data = [];

                array_push($table, $header);

                foreach ($cookies as $cookie) {
                    if ($cookie->trigger === 'Strictly Necessary Cookies') {
                        array_push($data, [
                            '0' => $cookie->link,
                            '1' => $cookie->zone,
                            '2' => $cookie->country,
                            '3' => $cookie->dgp,
                            '4' => $cookie->cookie,
                            '5' => $cookie->trigger,
                            '6' => $cookie->expected,
                            '7' => $cookie->status,
                        ]);
                    }
                }

                $data = $this->sortArray($data, array('1', '2', '0'));

                foreach ($data as $value) {
                    array_push($table, $value);
                }

                echo ($this->str_putcsv($table));
            } else if ($this->option('report-domain') && $this->option('report-domain') !== '?') {
                $domains = CookieComplianceReport::where('executed_at', $this->option('report-domain'))->get();

                $header = [
                    '0' => 'domain',
                    '1' => 'zone',
                    '2' => 'country',
                    '3' => 'site status',
                    '4' => 'DGP scope',
                    '5' => 'cookieBannerImplemented',
                    '6' => 'cookieDroppingCompliance',
                    '7' => 'number of cookies before consent (other than strictly necessary)',
                    '8' => 'script status',
                    '9' => 'banner implemented + script status',
                    '10' => 'compliance status',
                ];

                $table = [];
                $data = [];

                array_push($table, $header);

                foreach ($domains as $domain) {
                    $scriptStatus = OnetrustCookies::where('domain', 'ilike', $domain->link)->orWhere('domain', 'ilike', 'www.' . $domain->link)->first();

                    if (!(is_null($scriptStatus))) {
                        $scriptStatus = $scriptStatus['scriptStatus'];
                    } else {
                        $scriptStatus = null;
                    }

                    $mergeImplemented = ($domain->cookie_banner_implemented || ($scriptStatus === 'Published')) ? 'TRUE' : 'FALSE';

                    $domainCompliance = ($domain->cookie_compliance && $mergeImplemented === 'TRUE') ? 'TRUE' : 'FALSE';

                    array_push($data, [
                        '0' => $domain->link,
                        '1' => $domain->zone,
                        '2' => $domain->country,
                        '3' => $domain->site_status,
                        '4' => $domain->dgp,
                        '5' => ($domain->cookie_banner_implemented ? 'TRUE' : 'FALSE'),
                        '6' => ($domain->cookie_compliance ? 'TRUE' : 'FALSE'),
                        '7' => $domain->non_strictly,
                        '8' => $scriptStatus,
                        '9' => $mergeImplemented,
                        '10' => $domainCompliance,
                    ]);
                }

                $data = $this->sortArray($data, array('1', '2', '0'));

                foreach ($data as $value) {
                    array_push($table, $value);
                }

                echo ($this->str_putcsv($table));
            } else if ($this->option('crawl-bpoc') && $this->option('crawl-bpoc') !== '?') {
                $executedAt = $this->option('crawl-bpoc');

                $sites = BpocWebsiteList::where([
                    ['status', 'live'],
                    ['dgp_scope', true],
                ])->orderBy('link')->get();

                $runned = CookieComplianceReport::where('executed_at', $executedAt)->pluck('link')->toArray();

                foreach ($sites as $site) {
                    if (!in_array($site->link, $runned)) {
                        $this->call('crawler:cookie', [
                            'link' => $site->link,
                            'zone' => $site->zone,
                            'additionalCrawler' => $this->getAdditionalCrawler($site->link),
                            'executed_at' => $executedAt,
                            'official' => ($this->option('official') !== '?' ? $this->option('official') : 0),
                            'country' => $site->country,
                        ]);
                    } else {
                        $this->line('Skipping: ' . $site->link);
                    }
                }

                try {
                    app(CrawlerHelper::class)->sendNotification('Cookie Automation execution finished!', 'Cookie');

                    Artisan::call('optimize:clear');

                    apcu_clear_cache();
                } catch (Exception $e) {
                    $this->error($e->getMessage());
                }
            }
        } catch (Exception $e) {
            $this->error('Failed at ' . $e->getLine() . '. Error message: ' . $e->getMessage());
            $return = 1;
        } finally {
            return $return;
        }
    }

    /**
     * Retrieve Onetrust Access Token (oauth)
     *
     * @return ?stdClass
     */
    protected function initializeOnetrust(): ?stdClass
    {
        $account = null;

        try {
            $params = [
                'client_id' => env('ONETRUST_CLIENT_ID'),
                'client_secret' => env('ONETRUST_CLIENT_SECRET'),
                'grant_type' => 'client_credentials',
            ];

            $curl = curl_init();

            curl_setopt_array($curl, array(
                CURLOPT_URL => 'https://app-de.onetrust.com/api/access/v1/oauth/token',
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_ENCODING => '',
                CURLOPT_MAXREDIRS => 10,
                CURLOPT_TIMEOUT => 0,
                CURLOPT_FOLLOWLOCATION => true,
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => 'POST',
                CURLOPT_POSTFIELDS => http_build_query($params),
            ));

            $response = curl_exec($curl);

            curl_close($curl);

            $account = json_decode($response);

            if (isset($account->error)) {
                throw new Exception($account->error_description); // @phpstan-ignore-line
            }

            $this->accessToken = $account->access_token;
        } catch (Exception $e) {
            $this->error('Failed at ' . $e->getLine() . '. Error message: ' . $e->getMessage());

            $account = null;
        } finally {
            return $account;
        }
    }

    /**
     * Sort array with a given key
     *
     * @param array $data
     * @param string|array $field
     *
     * @return array
     */
    protected function sortArray(array $data, string|array $field): array
    {
        $field = (array) $field;

        uasort($data, function ($a, $b) use ($field) {
            $retval = 0;
            foreach ($field as $fieldname) {
                if ($retval === 0) $retval = strnatcmp($a[$fieldname], $b[$fieldname]);
            }
            return $retval;
        });

        return $data;
    }

    /**
     * Convert array to a CSV string
     *
     * @param array $data
     *
     * @return string
     */
    protected function str_putcsv($data): string
    {
        # Generate CSV data from array
        $fh = fopen('php://temp', 'rw');

        foreach ($data as $row) {
            fputcsv($fh, $row);
        }
        rewind($fh);
        $csv = stream_get_contents($fh);
        fclose($fh);

        return $csv;
    }

    /**
     * Update data inside OnetrustCookie Table
     *
     * @param bool $truncate
     *
     * @return void
     */
    protected function updateCookieInformation(bool $truncate = true, bool $fullUpdate = true): void
    {
        $websites = $this->getWebsites();

        $this->line('Getting websites');

        if ($truncate) {
            OnetrustCookies::truncate();
        }

        foreach ($websites as $site) {
            $published = '';

            $this->line('Getting cookies for ' . $site->domainName);
            $cookies = $this->getCookies($site->domainName);
            $this->line('Got ' . count($cookies));

            // For the usual update
            if ($fullUpdate) {
                $this->line('Getting script status for ' . $site->domainName);
                $published = $this->getScriptStatus($site->domainId);
            } else {
                // This case, we'll consider the script status that we already have
                $this->line('Skipping getting script status flow for ' . $site->domainName);

                try {
                    $currentData = OnetrustCookies::where('domain', $site->domainName)->firstOrFail();

                    $published = $currentData->scriptStatus;
                } catch (Exception $e) {
                    // In cases that we don't have any entry for the domain, lets get the script status
                    $this->line('Getting script status for ' . $site->domainName);
                    $published = $this->getScriptStatus($site->domainId);
                } finally {
                    OnetrustCookies::where('domain', $site->domainName)->delete();
                }
            }

            if (count($cookies) > 0) {
                foreach ($cookies as $cookie) {
                    $this->line('Handling with ' . $cookie->cookieName);

                    try {
                        $cookieCategory = $cookie->defaultCategory;
                        $this->line('Default category: ' . $cookieCategory);

                        if ($cookieCategory !== 'Strictly Necessary Cookies' && ((!(is_null($cookie->domainCookieInfoDtoList))) && count($cookie->domainCookieInfoDtoList) > 0)) {
                            $this->line('Have overwriting at the domain level');

                            foreach ($cookie->domainCookieInfoDtoList as $indexOverwriting => $cookieOverwriting) {
                                // TODO: Maybe in the future, we should always consider overwriting instead of only when unkwon
                                if (($cookieCategory === 'Unknown' || empty($cookieCategory)) && $indexOverwriting === 0) {
                                    $cookieCategory = $cookieOverwriting->displayGroupName;
                                    $this->line('Changed to the first entry: ' . $cookieCategory);
                                    continue;
                                }

                                if ($cookieOverwriting->displayGroupName === 'Strictly Necessary Cookies') {
                                    $cookieCategory = $cookieOverwriting->displayGroupName;
                                    $this->line('Changed to stricly');
                                    break;
                                }
                            }
                        }

                        $entry = new OnetrustCookies();

                        $entry->domain = $site->domainName;
                        $entry->domainId = $site->domainId;
                        $entry->cookieName = $cookie->cookieName;
                        $entry->lifespan = $cookie->lifespan;
                        $entry->cookie_host = $cookie->host;
                        $entry->defaultCategory = $cookieCategory;
                        $entry->cookieSource = $cookie->cookieSource;
                        $entry->thirdParty = $cookie->thirdParty;
                        $entry->expiry = $cookie->expiry;

                        $entry->scriptStatus = $published;

                        $this->line('Storing cookie');

                        $entry->save();
                    } catch (Exception $e) {
                        $this->error('Failed to store OnetrustCookie. Line: ' . $e->getLine() . '. Error message: ' . $e->getMessage());
                    }
                }
            } else {
                try {
                    $entry = new OnetrustCookies();

                    $entry->domain = $site->domainName;
                    $entry->domainId = $site->domainId;
                    $entry->cookieName = '';
                    $entry->lifespan = '';
                    $entry->cookie_host = '';
                    $entry->defaultCategory = '';
                    $entry->cookieSource = '';
                    $entry->thirdParty = '';
                    $entry->expiry = '';

                    $entry->scriptStatus = $published;

                    $entry->save();

                    $this->line('Stored empty entry');
                } catch (Exception $e) {
                    $this->error('Failed to store single OnetrustCookie. Line: ' . $e->getLine() . '. Error message: ' . $e->getMessage());
                }
            }
        }
    }
}
