<?php

// phpcs:disable PSR2.ControlStructures.ElseIfDeclaration.NotAllowed

namespace App\Console\Commands\Crawlers;

use App\Console\CrawlerHelper;
use Exception;
use Google\Cloud\Functions\V1\CloudFunctionsServiceClient;
use Illuminate\Console\Command;

/**
 * LighthouseCrawler
 *
 * Run Lighthouse on a specific link
 *
 * @author Lucas Buzzo <lucas.buzzo@ab-inbev.com>
 * @package App\Console\Commands\Crawlers
 */
class LighthouseCrawler extends Command
{
	/**
	 * The console command description.
	 */
	protected $description = 'Run Lighthouse tests on a specific link, with additional options (example.: bypass age gate)';

	/**
	 * The name and signature of the console command.
	 */
	protected $signature = 'crawler:lighthouse
        {link : Specify the link to run}
        {additionalCrawler : Additional Options to use on crawler}
        {acquiaHosted : Is the site hosted at Acquia?}
        {zone : Website\'s zone}
        {executed_at : Execution timestamp}
				{official : Define if this is an official execution}
				{ignoreInCriteria? : Ignore criteria for specific asset}	
        ';

	/**
	 * Create a new command instance.
	 *
	 * @return void
	 */
	public function __construct()
	{
		parent::__construct();
	}

	/**
	 * Run Lighthouse Crawler at GCP's CFs
	 *
	 * @param string $originalLink
	 * @param bool $acquiaHosted
	 * @param string $zone
	 * @param string $executedAt
	 *
	 * @return void
	 */
	public function crawl(string $originalLink, bool $acquiaHosted, string $zone, string $executedAt = '', $official = false): void
	{
		// Ensure the link is crawlable
		if (strpos($originalLink, 'http') !== 0) {
			$link = 'http://' . strtolower(trim($originalLink));
		} else {
			$link = strtolower(trim($originalLink));
		}

		try {
			// Go 4 leves up from the current directory
			$path = dirname(__DIR__, 4);

			$cloudFunctionsServiceClient = new CloudFunctionsServiceClient([
				'credentials' => json_decode(file_get_contents($path . '/' . env('GCP_CREDENTIALS_FILE')), true),
			]);

			$zone = $this->resolveRegion($zone, $acquiaHosted);

			// Define which cloud function run
			$formattedName = $cloudFunctionsServiceClient->cloudFunctionName(env('GCP_PROJECT'), $zone['region'], $zone['script']);

			// Used to encode as json and pass as as parameter in the request
			$data['link'] = $link;

			$this->line('Auditing with ' . $zone['region'] . '/' . $zone['script']);

			// Run the CF
			$response = $cloudFunctionsServiceClient->callFunction($formattedName, json_encode($data))->getResult();

			$json = json_decode($response);

			$fcp = 0;
			$lcp = 0;
			$speedIndex = 0;
			$tbt = 0;
			$tii = 0;
			$cls = 0;

			foreach ($json->audits as $key => $value) {
				switch ($key) {
					case 'first-contentful-paint':
						$fcp = $value->displayValue;
						break;
					case 'largest-contentful-paint':
						$lcp = $value->displayValue;
						break;
					case 'total-blocking-time':
						$tbt = $value->displayValue;
						break;
					case 'speed-index':
						$speedIndex = $value->displayValue;
						break;
					case 'interactive':
						$tii = $value->displayValue;
						break;
					case 'cumulative-layout-shift':
						$cls = $value->displayValue;
						break;
					default:
						break;
				}
			}

			$score = $json->categories->performance->score * 100;

			app(CrawlerHelper::class)->storeResult(
				link: $originalLink,
				criteria: 'first-contentful-paint',
				response: $fcp,
				executedAt: $executedAt,
				official: $official,
			);

			app(CrawlerHelper::class)->storeResult(
				link: $originalLink,
				criteria: 'largest-contentful-paint',
				response: $lcp,
				executedAt: $executedAt,
				official: $official,
			);

			app(CrawlerHelper::class)->storeResult(
				link: $originalLink,
				criteria: 'speed-index',
				response: $speedIndex,
				executedAt: $executedAt,
				official: $official,
			);

			app(CrawlerHelper::class)->storeResult(
				link: $originalLink,
				criteria: 'total-blocking-time',
				response: $tbt,
				executedAt: $executedAt,
				official: $official,
			);

			app(CrawlerHelper::class)->storeResult(
				link: $originalLink,
				criteria: 'time-to-interactive',
				response: $tii,
				executedAt: $executedAt,
				official: $official,
			);

			app(CrawlerHelper::class)->storeResult(
				link: $originalLink,
				criteria: 'cumulative-layout-shift',
				response: $cls,
				executedAt: $executedAt,
				official: $official,
			);

			app(CrawlerHelper::class)->storeResult(
				link: $originalLink,
				criteria: 'lighthouse',
				response: strval($score),
				executedAt: $executedAt,
				official: $official,
			);

			app(CrawlerHelper::class)->storeSourceCode(
				crawler: 'crawler:lighthouse',
				link: $originalLink,
				content: $response,
				failed: false,
				executedAt: $executedAt,
				official: $official,
			);
		} catch (Exception $e) {
			$this->error('Lighthouse Crawler Error for ' . $originalLink . '. Error message: ' . $e->getMessage());

			app(CrawlerHelper::class)->storeSourceCode(
				crawler: 'crawler:lighthouse',
				link: $originalLink,
				content: $e->getMessage(),
				failed: true,
				executedAt: $executedAt,
				official: $official,
			);
		}

		return;
	}

	/**
	 * Execute the console command.
	 *
	 * @return int
	 */
	public function handle(): int
	{
		$originalLink = $this->argument('link');

		// TODO: we need to at the bypass flow at CFs
		$additionalCrawler = $this->argument('additionalCrawler');

		$executedAt = $this->argument('executed_at');

		$acquiaHosted = $this->argument('acquiaHosted');
		$zone = $this->argument('zone');

		$official = ($this->argument('official') ? $this->argument('official') : false);

		$ignoreInCriteria = ($this->argument('ignoreInCriteria') ? $this->argument('ignoreInCriteria') : false);

		if (!$ignoreInCriteria) {
			$this->line('Lighthouse Auditing: ' . $originalLink);
			$this->crawl($originalLink, $acquiaHosted, $zone, $executedAt, $official);
		}

		return 0;
	}

	/** Run lighthouse locally
	 *
	 * @param string $originalLink
	 * @param string $additionalCrawler
	 * @param string $executedAt
	 *
	 * @return void
	 */
	protected function localCrawl($originalLink, $additionalCrawler, $executedAt = ''): void
	{
		try {
			// Ensure the link is crawlable
			if (strpos($originalLink, 'http') !== 0) {
				$link = 'http://' . $originalLink;
			} else {
				$link = $originalLink;
			}

			echo 'Auditing locally';

			exec('node app/Console/Js/lighthouse.js ' . $link . ' \'' . $additionalCrawler . '\' 2>&1', $output, $return_var);

			app(CrawlerHelper::class)->storeSourceCode(
				crawler: 'crawler:lighthouse',
				link: $originalLink,
				content: $output[0],
				failed: false,
				executedAt: $executedAt,
				official: false,
			);

			app(CrawlerHelper::class)->storeResult(
				link: $originalLink,
				criteria: 'lighthouse',
				response: json_decode($output[0])->score,
				executedAt: $executedAt,
				official: false,
			);
		} catch (Exception $e) {
			app(CrawlerHelper::class)->storeSourceCode(
				crawler: 'crawler:lighthouse',
				link: $originalLink,
				content: $e->getMessage(),
				failed: true,
				executedAt: $executedAt,
				official: false,
			);
		}

		return;
	}

	/**
	 * Resolve which script should be executed and where is located on GCP
	 *
	 * @param string $zone
	 * @param bool $acquiaHosted
	 *
	 * @return array
	 */
	public function resolveRegion(string $zone, bool $acquiaHosted = false): array
	{
		// TODO: we need to move CF's specifications from env to database
		if ($acquiaHosted || $zone === 'Europe') {
			$region = env('GCP_CF_LIGHTHOUSE_EUR_REGION', '');
			$script = env('GCP_CF_LIGHTHOUSE_EUR_SCRIPT', '');
		} else if ($zone === 'Asia Pacific') {
			$region = env('GCP_CF_LIGHTHOUSE_APAC_REGION', '');
			$script = env('GCP_CF_LIGHTHOUSE_APAC_SCRIPT', '');
		} else {
			$region = env('GCP_CF_LIGHTHOUSE_NAZ_REGION', '');
			$script = env('GCP_CF_LIGHTHOUSE_NAZ_SCRIPT', '');
		}

		return array(
			'region' => $region,
			'script' => $script
		);
	}
}
