<?php

namespace App\Console\Commands\Crawlers;

use App\Console\CrawlerHelper;
use App\Models\Links;
use Exception;
use Google\Cloud\BigQuery\BigQueryClient;
use Illuminate\Console\Command;

/**
 * Site Crawler
 *
 * Command to run site crawler on a specific link
 *
 * @author Lucas Buzzo <lucas.buzzo@ab-inbev.com>
 * @package App\Console\Commands\Crawlers
 */
class SemrushCrawler extends Command
{
    /**
     * The console command description.
     */
    protected $description = 'Update semrush informations';

    /**
     * Keep all the current DGP links
     *
     * @var array
     */
    protected array $dgpLinks = [];

    /**
     * Keep all the current SemRush Data
     *
     * @var array
     */
    protected array $semrushData = [];

    /**
     * The name and signature of the console command.
     */
    protected $signature = 'crawler:semrush
    {executed_at : Define the execution date}
    {official? : Define if is an official execution}';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Retrieve Semrush informations from BQ
     *
     * @return void
     */
    protected function getData(): void
    {
        $data = [];
        try {
            // Go 4 leves up from the current directory
            $path = dirname(__DIR__, 4);

            $big = new BigQueryClient([
                'keyFile' =>
                json_decode(file_get_contents($path . '/' . env('GCP_CREDENTIALS_FILE')), true),
            ]);

            $queryJobConfig = $big->query(
                'SELECT url, site_health, execution_time FROM `abi-martech-global-wa.sem_rush.rw_projects_data` WHERE execution_time = (SELECT MAX(execution_time) FROM `abi-martech-global-wa.sem_rush.rw_projects_data`)',
            );

            $queryResults = $big->runQuery($queryJobConfig);

            foreach ($queryResults as $value) {
                array_push($data, [
                    'url' => $value['url'],
                    'score' => $value['site_health'],
                    'execution_time' => $value['execution_time']->get()->format('Y-m-d H:i:s'),
                ]);
            }
        } catch (Exception $e) {
            $data = [];
            $this->error('Failed at Semrush crawler\' getData function. Error message: ' . $e->getMessage());
        } finally {
            $this->semrushData = $data;
        }
    }

    /**
     * Retrieve the SemRush data for a given domain
     *
     * @param string $link
     *
     * @return ?array
     */
    protected function getInfoForDomain(string $link): ?array
    {
        $result = null;

        $url = strtolower(trim(ltrim($link, 'www.')));

        foreach ($this->semrushData as $value) {
            $domain = strtolower(trim(ltrim($value['url'], 'www.')));

            if ($domain === $url) {
                $result = $value;

                break;
            }
        }

        return $result;
    }

    /**
     * Execute the console command.
     *
     * @return int
     */
    public function handle(): int
    {
        $executedAt = $this->argument('executed_at');

        $official = ($this->argument('official') ? $this->argument('official') : false);

        $this->getData();

        $this->initializeLinks();

        foreach ($this->dgpLinks as $dgpLink) {
            if ($dgpLink['country'] === 'Ukraine' || $dgpLink['country'] === 'Russia') continue;

            $data = $this->getInfoForDomain($dgpLink['link']);

            try {
                if (is_null($data)) {
                    continue;
                    // throw new Exception('Failed to find link at SemRush data');
                }

                app(CrawlerHelper::class)->storeResult(
                    link: $dgpLink['link'],
                    criteria: 'semrush',
                    response: $data['score'],
                    executedAt: $executedAt,
                    official: $official,
                );
            } catch (Exception $e) {
                $this->error('Error at SemRush Crawler for ' . $dgpLink['link'] . '. Error message: ' . $e->getMessage());

                app(CrawlerHelper::class)->storeSourceCode(
                    crawler: 'crawler:semrush',
                    link: $dgpLink['link'],
                    content: $e->getMessage(),
                    failed: true,
                    executedAt: $executedAt,
                    official: $official,
                );
            }
        }

        return 0;
    }

    /**
     * Retrieve all current DGP links
     *
     * @return void
     */
    protected function initializeLinks(): void
    {
        $links = [];

        try {
            $data = Links::join('countries', 'links.country_id', '=', 'countries.id')->where([
                ['links.consider', true],
                ['links.channel_id', 110],
            ])->get(['links.link as link', 'countries.name as country']);

            foreach ($data as $link) {
                array_push($links, ['link' => $link->link, 'country' => $link->country]);
            }
        } catch (Exception $e) {
            $this->error('SemRush: Failed to retrieve DGP site list. Error message: ' . $e->getMessage());

            $links = [];
        } finally {
            $this->dgpLinks = $links;
        }
    }
}
