<?php

// phpcs:disable BestIt.DocTags.RequiredPropertyTags.TagOccurrenceMinVar
// phpcs:disable BestIt.TypeHints.PropertyTypeHint.MissingAnyTypeHint
// phpcs:disable PSR2.ControlStructures.ElseIfDeclaration.NotAllowed

namespace App\Console\Commands\Crawlers;

use App\Console\CrawlerHelper;
use Exception;
use GuzzleHttp\Client;
use Illuminate\Console\Command;

/**
 * YouTube Checker
 *
 * Command to check the complaice with implemented YouTube Accounts
 *
 * @author Lucas Buzzo <lucas.buzzo@ab-inbev.com>
 * @package App\Console\Commands\Crawlers
 */

class YouTubeChecker extends Command
{
	/**
	 * Run on debug mode
	 */
	protected bool $debug = false;

	/**
	 * The console command description.
	 */
	protected $description = 'Run YouTube IARDs and check compliance';

	/**
	 * List of iards needed to be tested
	 */
	protected $iards = ['coi', 'fan', 'rdm', 'ugc'];

	/**
	 * The signature to call the command
	 */
	protected $signature = 'crawler:youtube
        {link : Link to be checked}
        {locale : Define keywords to use}
        {executed_at : Execution timestamp}
				{official : Define if this is an official execution}
        {--debug : Dump responses instead of store}';

	/**
	 * Create a new command instance.
	 *
	 * @return void
	 */
	public function __construct()
	{
		parent::__construct();
	}

	/**
	 * Based on a channel link, check if there are age gate
	 *
	 * @param string $link
	 *
	 * @return bool
	 */
	protected function checkAgegate(string $link): bool
	{
		$agegate = false;
		$content = '';

		try {
			//dd('node app/Console/Js/browser.js ' . $link . '{} false');

			$output = shell_exec('node app/Console/Js/browser.js ' . $link . '{} false');

			$content = json_decode(trim($output), false, 512, JSON_INVALID_UTF8_IGNORE)->sourceCode;
		} catch (Exception $e) {
			$this->error('YouTube error checkAgegate. Message: ' . $e->getMessage());
		}

		if (strpos($content, 'channelAgeGateRenderer')) {
			$agegate = true;
		}

		return $agegate;
	}

	/**
	 * Based on a video url, check if channel has verified badge
	 *
	 * @param string $videoUrl
	 *
	 * @return bool
	 */
	protected function checkVerified(string $videoUrl): bool
	{
		$youtubeContent = '';

		try {
			$youtubeContent = trim(shell_exec('node app/Console/Js/YoutubeCrawler.js ' . $videoUrl));
		} catch (Exception $e) {
			$this->error('YouTube error checkVerified. Message: ' . $e->getMessage());
		}

		$verified = (json_decode($youtubeContent, true, 512, JSON_INVALID_UTF8_IGNORE))['verified'];

		return $verified;
	}

	/**
	 * For a given url, extract channel identifier
	 *
	 * @param string $link
	 *
	 * @return string
	 */
	protected function extractChannelFromUrl(string $link): string
	{
		$data = '';
		$channel = '';

		if (strpos($link, 'channel/') || strpos($link, 'user/')) {
			$channel = explode('/', $link)[4];
		} else {
			// We'll need crawl the page to discover the id
			try {
				$data = shell_exec('node app/Console/Js/YoutubeCrawler.js ' . $link);
			} catch (Exception $e) {
				$this->error('YouTube error extractChannelFromUrl. Message: ' . $e->getMessage());
			}

			$data = json_decode(trim($data), true, 512, JSON_INVALID_UTF8_IGNORE);

			if (isset($data['channelId'])) {
				$channel = $data['channelId'];
			}
		}

		return $channel;
	}

	/**
	 * For a given link, return channel id and description
	 *
	 * @param string $link
	 *
	 * @return array
	 */
	protected function getDetails(string $link): array
	{
		$result = [];

		// Extract channel identifier from url
		$channel = $this->extractChannelFromUrl($link);

		$apikey = env('YOUTUBE_GCP_KEY', '');

		$url = '';

		if (strpos($link, 'user') !== false) {
			$url = 'https://www.googleapis.com/youtube/v3/channels?forUsername=' . $channel . '&key=' . $apikey . '&part=brandingSettings,contentDetails';
		} else {
			$url = 'https://www.googleapis.com/youtube/v3/channels?id=' . $channel . '&key=' . $apikey . '&part=brandingSettings,contentDetails';
		}

		$descriptionClient = new Client();

		$response = $descriptionClient->get($url)->getBody();

		$data = json_decode($response, true);

		if ($data['pageInfo']['totalResults'] > 0) {
			$data = (json_decode($response, true))['items'][0];

			$id = $data['id'];

			if (isset($data['brandingSettings']['channel']['description'])) {
				$description = $data['brandingSettings']['channel']['description'];
			} else {
				$description = '';
			}

			$uploaded = $data['contentDetails']['relatedPlaylists']['uploads'];

			$result = array(
				'id' => $id,
				'description' => $description,
				'uploadsPlaylist' => $uploaded,
			);
		}

		return $result;
	}

	/**
	 * For a given channel's id return last posted video url
	 *
	 * @param string $channelId
	 *
	 * @return string
	 */
	protected function getLastVideoUrl(string $channelId): string
	{
		$apikey = env('YOUTUBE_GCP_KEY', '');

		$url = 'https://www.googleapis.com/youtube/v3/search?key=' . $apikey . '&channelId=' . $channelId . '&part=snippet,id&order=date&maxResults=1';

		$client = new Client();

		$response = $client->get($url)->getBody();

		$data = json_decode($response, true)['items'][0];

		$videoId = $data['id']['videoId'];

		return 'https://www.youtube.com/watch?v=' . $videoId;
	}

	/**
	 * For a givnen playlist, return first video url
	 *
	 * @param string $playlist
	 *
	 * @return string
	 */
	protected function getVideoFromPlaylist(string $playlist): string
	{
		$result = '';

		$apikey = env('YOUTUBE_GCP_KEY', '');

		$url = 'https://www.googleapis.com/youtube/v3/playlistItems?key=' . $apikey . '&playlistId=' . $playlist . '&part=snippet&maxResults=1';

		$client = new Client([
			'http_errors' => false,
		]);

		$request = $client->get($url);

		if ($request->getStatusCode() != 404) {
			$response = $client->get($url)->getBody();

			$data = json_decode($response, true)['items'][0];

			$videoId = $data['snippet']['resourceId']['videoId'];
			$result = 'https://www.youtube.com/watch?v=' . $videoId;
		}

		return $result;
	}

	/**
	 * Execute the console command.
	 *
	 * @return int
	 */
	public function handle(): int
	{
		if ($this->option('debug')) {
			$this->debug = true;
		}

		$retorno = 0;

		$link = $this->argument('link');
		$originalLink = $link;

		$locale = $this->argument('locale');

		$executedAt = $this->argument('executed_at');

		$official = ($this->argument('official') ? $this->argument('official') : false);

		if ($this->debug) {
			echo $link . "\t";
		}

		// Ensure site have www prefix
		if (strpos($link, 'www.') === false) {
			$link = 'www.' . $link;
		}

		// Handle with links without protocol
		if (strpos($link, 'http') === false) {
			$link = 'https://' . $link;
		}

		$link = explode('?', $link)[0];

		try {
			$details = $this->getDetails($link);
			// sleep(0.5);

			if (count($details) > 0) {
				$description = $details['description'];
				$uploaded = $details['uploadsPlaylist'];

				if (!($this->debug)) {
					app(CrawlerHelper::class)->storeSourceCode(
						crawler: 'crawler:youtube',
						link: $originalLink,
						content: $description,
						failed: false,
						executedAt: $executedAt,
						official: $official,
					);

					app(CrawlerHelper::class)->storeResult(
						link: $originalLink,
						criteria: 'http-response',
						response: '200',
						executedAt: $executedAt,
						official: $official,
					);
				}

				$ageGate = $this->checkAgegate($link);

				if ($this->debug) {
					echo 'agegate: ' . (int) $ageGate . "\t";
				} else {
					if ($ageGate) {
						app(CrawlerHelper::class)->storeResult(
							link: $originalLink,
							criteria: 'agegate',
							response: 'true',
							executedAt: $executedAt,
							official: $official,
						);
					} else {
						app(CrawlerHelper::class)->storeResult(
							link: $originalLink,
							criteria: 'agegate',
							response: 'false',
							executedAt: $executedAt,
							official: $official,
						);
					}
				}

				$lastVideoUrl = $this->getVideoFromPlaylist($uploaded);
				// sleep(0.5);

				$verified = 0;
				if (!(empty($lastVideoUrl))) {
					$verified = $this->checkVerified($lastVideoUrl);
				}

				if ($this->debug) {
					echo 'verified: ' . (int) $verified . "\t";
				}

				$this->runIardsOnAbout($description, $verified, $locale, $originalLink, $executedAt, $official);

				// sleep(1);
			} else {
				if (!($this->debug)) {
					app(CrawlerHelper::class)->storeSourceCode(
						crawler: 'crawler:youtube',
						link: $originalLink,
						content: '',
						failed: true,
						executedAt: $executedAt,
						official: $official,
					);

					app(CrawlerHelper::class)->storeResult(
						link: $originalLink,
						criteria: 'http-response',
						response: '0',
						executedAt: $executedAt,
						official: $official,
					);
				}

				throw new Exception("Channel don't exist");
			}

			if ($this->debug) {
				echo "\n";
			}
		} catch (Exception $e) {
			echo $e->getMessage();
			echo "\n";
			$retorno = 1;
		}

		return $retorno;
	}

	/**
	 * For a giver About text, run iards tests
	 *
	 * @param string $about
	 * @param bool $coi
	 * @param string $locale
	 * @param string $link
	 * @param string $executedAt
	 * @param bool $official
	 *
	 * @return void
	 */
	protected function runIardsOnAbout(string $about, bool $coi, string $locale, string $link, string $executedAt = '', bool $official = false): void
	{
		foreach ($this->iards as $iard) {
			$result = 0;

			try {
				$kws = CrawlerHelper::keywords($iard, $locale, true);

				if (count($kws) === 0) {
					throw new Exception('There are no locale keywords for this link');
				}

				if (is_array($kws)) {
					// If is an associative array, we'll define the elements as the first position
					if (array_key_first($kws) !== 0) {
						$kws = ($kws[array_key_first($kws)]);
					} else if (is_array($kws[0])) {
						$kws = $kws[0];
					}

					foreach ($kws as $index => $kw) {
						if (is_array($kw)) {
							$kws = array_merge($kws, $kws[$index]);
							unset($kws[$index]);
						}
					}
				}
			} catch (Exception $e) {
				$this->error('YouTube Crawler failed at keywords. Message: ' . $e->getMessage());
				break;
			}

			switch ($iard) {
				case 'coi':
				case 'rdm':
				case 'fan':
					$found = 0;
					foreach ($kws as $keyword) {
						if ($found === 0) {
							$found = (strpos(mb_strtolower(str_replace('<BR>', ' ', str_replace('<br>', ' ', $about))), mb_strtolower($keyword)) !== false) ? 1 : 0;

							if ($found) {
								break;
							}
						}
					}
					if ($iard === 'coi' && $coi) {
						$result = 'true';
					} else {
						if ($found === 1) {
							$result = 'true';
						}
					}
					break;
				case 'ugc':
					$found = false;

					// Explode new lines (turn into array)
					$text = app(CrawlerHelper::class)->splitNl($about);

					foreach ($text as $key => $line) {
						$down = mb_strtolower($line);

						if (!$found) {
							foreach ($kws as $kw) {
								$found = str_contains($down, mb_strtolower($kw));

								if ($found) {
									$urls = app(CrawlerHelper::class)->extractUrlsFromText($text[$key]);

									if (is_array($urls) && count($urls) > 0) {
										$status = app(CrawlerHelper::class)->getHttpCode($urls[0]);

										if ($status === 200) {
											$found = true;
											$result = 'true';
											break;
										} else {
											$found = false;
											$result = 'false';
											break;
										}
									} else {
										$found = true;
										$result = 'true';
										break;
									}
								}
							}
						} else {
							break;
						}
					}

					break;
			}

			if ($this->debug) {
				echo $iard . ': ' . $result . "\t";
			} else {
				app(CrawlerHelper::class)->storeResult(
					link: $link,
					criteria: $iard,
					response: $result,
					executedAt: $executedAt,
					official: $official,
				);
			}
		}
	}
}
