<?php

namespace App\Http\Controllers;

use App\Models\Channel;
use Exception;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Http\JsonResponse;
use Illuminate\Support\Str;

/**
 * Channel Controller
 *
 * @author Lucas Buzzo <lucas.buzzo@ab-inbev.com>
 * @package App\Http\Controllers
 */
class ChannelController extends Controller
{
    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     *
     * @return JsonResponse
     */
    public function destroy(int $id): JsonResponse
    {
        try {
            Channel::destroy($id);
            $return = [
                'status' => 'success',
                'message' => __('Deleted successfully!'),
            ];
            $statusCode = 201;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to delete!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Generate the slug for a given name
     *
     * @param string $name
     *
     * @return string
     */
    public function generateSlug(string $name): string
    {
        // Slugify
        $slug = Str::slug($name, '-');

        // Retrieve the last slug to the name
        $exist = app(ChannelController::class)->getLastChannelBySlug($slug);

        // If exist already exist a slug, get it
        if (!(is_null($exist))) {
            $exist = $exist->slug;
        }

        //Retrieve the last position for this separator
        $counter = last(explode('-', $exist));

        /**
         * If the last array position is a number, means that there are another entry to this name
         * In another hand, if is empty meands that is the first one.
         */
        if (is_numeric($counter)) {
            $counter += 1;
        } elseif (!(empty($counter))) {
            $counter = 1;
        }

        // If there are another one, increase the counter - to ensure that slug is unique
        if ($counter) {
            $slug = $slug . '-' . $counter;
        }

        return $slug;
    }

    /**
     * Return last Channel to a given slug
     *
     * @param string $slug
     *
     * @return Channel|null
     */
    public function getLastChannelBySlug(string $slug): ?Channel
    {
        return Channel::withTrashed()->where('slug', $slug)->orWhere('slug', '~', '^' . $slug . '-[0-9]+')->orderBy('id', 'DESC')->first();
    }

    /**
     * Display a listing of the resource.
     *
     * @return JsonResponse
     */
    public function index(): JsonResponse
    {
        try {
            $channels = Channel::all();
            $return = [
                'status' => 'success',
                'data' => $channels,
            ];
            $statusCode = 200;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to get!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Display a paginated listing of the resource.
     *
     * @return JsonResponse
     */
    public function index_paginated(): JsonResponse
    {
        try {
            $channels = Channel::paginate(env('PAGINATE_PER_PAGE', 20));
            $return = [
                'status' => 'success',
                'data' => $channels,
            ];
            $statusCode = 200;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to get!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Show the channel for a given id.
     *
     * @param int $id
     *
     * @return JsonResponse
     */
    public function show(int $id): JsonResponse
    {
        try {
            $channel = Channel::find($id);
            $return = [
                'status' => 'success',
                'data' => $channel,
            ];
            $statusCode = 200;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to get!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param string $params
     *
     * @return JsonResponse
     */
    public function store(string $params): JsonResponse
    {
        $params = json_decode($params, true);

        try {
            $channel = new Channel();
            $channel->name = $params['name'];
            $channel->slug = $params['slug'];

            if (isset($params['icon'])) {
                $channel->icon = $params['icon'];
            }

            $channel->save();

            $return = [
                'status' => 'success',
                'message' => __('Created successfully'),
            ];
            $statusCode = 201;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to Create!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param int $id
     * @param string $params
     *
     * @return JsonResponse
     */
    public function update(int $id, string $params): JsonResponse
    {
        $params = json_decode($params, true);

        try {
            $channel = Channel::find($id);

            if (!empty($params['name'])) {
                $channel->name = $params['name'];
            }

            if (!empty($params['icon'])) {
                $channel->icon = $params['icon'];
            }

            $channel->save();

            $return = [
                'status' => 'success',
                'message' => __('Updated successfully'),
            ];

            $statusCode = 200;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to Update!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }
}
