<?php

// phpcs:disable PSR2.ControlStructures.ElseIfDeclaration.NotAllowed

namespace App\Http\Controllers;

use App\Exports\ReportExport;
use App\Models\CookieComplianceReport;
use App\Models\CookiesWebsite;
use DateTime;
use Exception;
use Illuminate\Contracts\Database\Eloquent\Builder;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Cache;
use Maatwebsite\Excel\Facades\Excel;
use Symfony\Component\HttpFoundation\BinaryFileResponse;

/**
 * CookieReport Controller
 *
 * @author Lucas Buzzo <lucas.buzzo@ab-inbev.com>
 * @package App\Http\Controllers
 */

class CookieReportController extends Controller
{
    /**
     * Retrieve the domain Level Report
     *
     * @param Request $request
     *
     * @return JsonResponse
     */
    public function domainLevelReport(Request $request): JsonResponse
    {
        $return = [];
        $statusCode = 500;

        try {
            $query = $this->getData($request);

            if (is_null($query)) {
                throw new Exception('No data found');
            }

            $report = $query->paginate($request->per_page, [
                'zone',
                'country',
                'dgp',
                'link',
                'executed_at',
                'site_status',
                'cookie_banner_implemented',
                'cookie_compliance',
                'non_strictly',
                'domain_compliance',
                'scriptStatus',
            ]);

            $excutedAt = null;

            if (count($report) > 0) {
                // @phpstan-ignore-next-line
                $excutedAt = $report[0]->executed_at;
            }

            $return = [
                'status' => 'success',
                'data' => $report,
                'lastRun' => $excutedAt,
            ];

            $statusCode = 200;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => 'Failed to retrieve domainLevelReport',
                'details' => $e->getMessage(),
            ];

            $statusCode = 500;
        } finally {
            return response()->json($return, $statusCode);
        }
    }

    /**
     * Export Domain Level Report
     *
     * @param Request $request
     *
     * @return BinaryFileResponse|JsonResponse
     */
    public function exportReport(Request $request): BinaryFileResponse|JsonResponse
    {
        $return = [];

        try {
            $data = $this->getData($request)->get();

            if (count($data) === 0) {
                throw new Exception('No data found');
            }

            $table[0] = array(
                'Domain',
                'Zone',
                'Country',
                'Site Status',
                'DGP Scope',
                'Banner Implemented',
                'Cookie Dropping Compliance',
                'Cookies that need attention',
                'Domain Compliance'
            );

            foreach ($data as $value) {
                $index = count($table);

                $implemented = null;

                if ($value->scriptStatus === 'Published' || $value->cookie_banner_implemented) { // @phpstan-ignore-line
                    $implemented = 'TRUE';
                } else if (!is_null($value->cookie_banner_implemented)) {
                    $implemented = 'FALSE';
                } else {
                    $implemented = 'NA';
                }

                $cookieCompliance = null;

                if (is_null($value->cookie_compliance)) { // @phpstan-ignore-line
                    $cookieCompliance = 'NA';
                } else if ($value->cookie_compliance) {
                    $cookieCompliance = 'TRUE';
                } else {
                    $cookieCompliance = 'FALSE';
                }

                $table[$index] = array(
                    $value->link, // @phpstan-ignore-line
                    $value->zone, // @phpstan-ignore-line
                    $value->country, // @phpstan-ignore-line
                    $value->site_status, // @phpstan-ignore-line
                    ($value->dgp ? 'TRUE' : 'FALSE'), // @phpstan-ignore-line
                    $implemented,
                    $cookieCompliance,
                    (string) $value->non_strictly, // @phpstan-ignore-line
                    ($value->domain_compliance ? 'TRUE' : 'FALSE'), // @phpstan-ignore-line
                );
            }

            $report = new ReportExport($table);

            $date = $data[0]->executed_at; // @phpstan-ignore-line

            $date = new DateTime($date);

            $filename = $date->format('Ymd') . '-domain-level.xlsx';

            $return = Excel::download($report, $filename);
        } catch (Exception $e) {
            $return = response()->json([
                'status' => 'error',
                'message' => 'Failed to download report',
                'detail' => $e->getMessage(),
            ]);
        } finally {
            return $return;
        }
    }

    /**
     * Export Website Cookie List
     *
     * @param Request $request
     *
     * @return BinaryFileResponse|JsonResponse|null
     */
    public function exportWebsiteCookies(Request $request): BinaryFileResponse|JsonResponse|null
    {
        $return = null;

        try {
            $zones = [];

            if (!(empty($request->zones))) {
                foreach ($request->zones as $value) {
                    if (is_array($value)) {
                        $decoded = $value['text'];
                    } else {
                        $decoded = json_decode($value);
                        $decoded = $decoded->text;
                    }

                    array_push($zones, $decoded);
                }
            }

            $dgpScope = [];

            if (!(empty($request->dgp_goal))) {
                foreach ($request->dgp_goal as $value) {
                    if (is_array($value)) {
                        $decoded = $value['value'];
                    } else {
                        $decoded = json_decode($value);
                        $decoded = $decoded->value;
                    }

                    array_push($dgpScope, $decoded);
                }
            }

            $query = CookiesWebsite::query();

            $query->when(!empty($request->zones), function ($q) use ($zones) {
                return $q->whereIn('zone', $zones);
            });

            $query->when(!empty($request->dgp_goal), function ($q) use ($dgpScope) {
                return $q->whereIn('dgp', $dgpScope);
            });

            $query->when(!empty($request->siteStatus), function ($q) {
                return $q->where('site_status', request('siteStatus'));
            });

            $query->when(!empty($request->text), function ($q) {
                return $q->where('link', 'ilike', '%' . request('text') . '%');
            });

            $query->orderByRaw('UPPER(zone) ASC, UPPER(country) ASC, UPPER(link) ASC');

            $data = $query->get([
                'link',
                'zone',
                'country',
                'dgp',
                'cookie',
                'trigger',
                'expected',
                'status',
            ]);

            if (count($data) === 0) {
                throw new Exception('No data found');
            }

            $table[0] = array(
                'Domain',
                'Zone',
                'Country',
                'DGP Scope',
                'Cookie Name',
                'Dropping as',
                'Current OT Category',
                'Status',
                'Actions',
            );

            foreach ($data as $value) {
                $index = count($table);

                $table[$index] = array(
                    $value->link,
                    $value->zone,
                    $value->country,
                    ($value->dgp ? 'TRUE' : 'FALSE'),
                    $value->cookie,
                    $value->trigger,
                    $value->expected,
                    $value->status,
                    $this->getActionForStatus($value->status),
                );
            }

            $report = new ReportExport($table);

            $date = new DateTime();

            $filename = $date->format('Ymd') . '-cookie-level.xlsx';

            $return = Excel::download($report, $filename);
        } catch (Exception $e) {
            $return = response()->json([
                'status' => 'error',
                'message' => 'Failed to download report',
                'detail' => $e->getMessage(),
            ]);
        } finally {
            return $return;
        }
    }

    /**
     * Return the needed actions for a given cookie status
     *
     * @param string $status
     *
     * @return string
     */
    protected function getActionForStatus(string $status): string
    {
        $actions = '';

        switch ($status) {
            case 'Uncategorized cookie':
                $actions = '1) Check if the cookie is indeed strictly needed, otherwise fix it on the website; 2) Category the cookie in the OneTrust';
                break;
            case 'Different categorization':
                $actions = '1) Check if the cookie is indeed strictly needed, otherwise fix it on the website; 2) Ensure the way the website drops the cookie is the same as categorized in OT';
                break;
        }

        return $actions;
    }

    /**
     * Get filters data
     *
     * @return JsonResponse
     */
    public function getCookieFilterData(): JsonResponse
    {
        $statusCode = 0;
        $return = [];

        try {
            // $countries = $this->getDistinctValue('country');

            $officialDates = $this->getDistinctValue('executed_at');

            $zones = $this->getDistinctValue('zone');

            $return = [
                'status' => 'success',
                'data' => array(
                    // 'countries' => $countries,
                    'officialDates' => $officialDates,
                    'zones' => $zones,
                ),
            ];

            $statusCode = 200;
        } catch (Exception $e) {
            $statusCode = 500;

            $return = [
                'status' => 'error',
                'message' => 'Failed to retrieve filters',
                'detail' => $e->getMessage(),
            ];
        } finally {
            return response()->json($return, $statusCode);
        }
    }

    /**
     * For a given domain, receive the cookie related data (Created for BPOC team)
     *
     * @param Request $request
     *
     * @return JsonResponse
     */
    public function getCookieRelatedData(Request $request): JsonResponse
    {
        $statusCode = 0;
        $return = [];
        $data = [];

        try {
            $data = CookieComplianceReport::query();

            $data->where('link', 'ilike', $request->link);

            if (!empty($request->executedAt)) {
                $data->where('executed_at', $request->executedAt);
            }

            $data->orderBy('executed_at', 'desc');

            $info = $data->take(1)->get()[0];

            $implemented = ($info->cookie_banner_implemented || $info->scriptStatus === 'Published') ? true : false;

            $return = [
                'status' => 'success',
                'data' => [
                    'domain' => $info->link,
                    'executed_at' => $info->executed_at,
                    'cookie_banner_implemented' => $implemented,
                    'cookie_compliance' => $info->cookie_compliance,
                ],
            ];

            $statusCode = 200;
        } catch (Exception $e) {
            $statusCode = 500;

            $return = [
                'status' => 'failed',
                'message' => $e->getMessage(),
            ];
        } finally {
            return response()->json($return, $statusCode);
        }
    }

    /**
     * For the given filters return data
     *
     * @param Request $request
     *
     * @return Builder|null
     */
    protected function getData(Request $request): ?Builder
    {
        $data = null;

        try {
            $date = null;

            if (isset($request->date)) {
                $date = $request->date;
            }

            $zones = [];

            if (!(empty($request->zones))) {
                foreach ($request->zones as $value) {
                    if (is_array($value)) {
                        $decoded = $value['text'];
                    } else {
                        $decoded = json_decode($value);
                        $decoded = $decoded->text;
                    }

                    array_push($zones, $decoded);
                }
            }

            $dgpScope = [];

            if (!(empty($request->dgp_goal))) {
                foreach ($request->dgp_goal as $value) {
                    if (is_array($value)) {
                        $decoded = $value['value'];
                    } else {
                        $decoded = json_decode($value);
                        $decoded = $decoded->value;
                    }

                    array_push($dgpScope, $decoded);
                }
            }

            if ($date === null) {
                $lastRun = CookieComplianceReport::orderBy('executed_at', 'desc')->first();

                $date = $lastRun->executed_at;
            }

            $query = CookieComplianceReport::query();

            $query->whereDate('executed_at', $date);

            $query->when(!empty($request->zones), function ($q) use ($zones) {
                return $q->whereIn('zone', $zones);
            });

            $query->when(!empty($request->dgp_goal), function ($q) use ($dgpScope) {
                return $q->whereIn('dgp', $dgpScope);
            });

            $query->when(!empty($request->siteStatus), function ($q) {
                return $q->where('site_status', request('siteStatus'));
            });

            $query->when(!empty($request->text), function ($q) {
                return $q->where('link', 'ilike', '%' . request('text') . '%');
            });

            $query->orderByRaw('UPPER(zone) ASC, UPPER(country) ASC, UPPER(link) ASC');

            $data = $query;
        } catch (Exception) {
            $data = null;
        } finally {
            return $data;
        }
    }

    /**
     * For a given column return the distinct values
     *
     * @param string $column
     *
     * @return array
     */
    protected function getDistinctValue(string $column): array
    {
        $return = [];

        try {
            if (Cache::has('cookie-filter-' . $column)) {
                $return = Cache::get('cookie-filter-' . $column);
            } else {
                $return = CookieComplianceReport::distinct($column)->pluck($column)->toArray();

                Cache::put('cookie-filter-' . $column, $return, now()->addMinutes(120));
            }
        } catch (Exception) {
            $return = [];
        } finally {
            return $return;
        }
    }
}
