<?php

namespace App\Http\Controllers;

use App\Http\Controllers\LinkCriteriaController;
use App\Imports\BrandProtectionPlatformImport;
use App\Models\Brand;
use App\Models\Channel;
use App\Models\Country;
use App\Models\Links;
use App\Models\Ownership;
use Exception;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Maatwebsite\Excel\Facades\Excel;

ini_set('memory_limit', '24095M');
ini_set('max_execution_time', '0');
set_time_limit(0);


/**
 * ImportBpController Controller
 *
 * @author Tércio Lacerda <Tercio.lacerda-ext@ab-inbev.com>
 * @package App\Http\Controllers
 */
class ImportBpController extends Controller
{
    /**
     * Check if request has required params
     *
     * @param Array $params
     *
     * @return Array
     */
    protected static function checkRequiredParams(array $params): array
    {
        $requiredParams = [
            'domain',
            'assetType',
            'assetTypeColumn',
            'country',
            'brand',
            'consider',
            'environment',
            'status',
            'scope',
            'businessArea',
            'assessments',
            'executed_at',
        ];

        $missingParams = array();

        foreach ($requiredParams as $key => $value) {
            if (empty($params[$value]) || strlen($params[$value]) === 0) {
                array_push($missingParams, $value);
            }
        }

        return [
            'result' => (count($missingParams) === 0),
            'missingParams' => implode(', ', $missingParams),
        ];
    }

    /**
     * Convert Excel column to array index
     *
     * @param String $column
     *
     * @return int
     */
    protected static function columnToIndex(String $column): int
    {
        $number = 0;
        foreach (str_split($column) as $letter) {
            $number = ($number * 26) + (ord(strtolower($letter)) - 96);
        }
        return ($number - 1);
    }

    /**
     * Save BP Import
     *
     * @param Request $request
     *
     * @return JsonResponse
     */
    public function importBpHandler(Request $request): JsonResponse
    {
        try {
            $errors = array(
                'country' => [],
            );

            $params = $request->all();
            $checkParams = self::checkRequiredParams($params);
            if (!$checkParams['result']) {
                throw new Exception('Requireds params: ' . $checkParams['missingParams'] . ' are missing.', 1);
            }

            $file = $request->file('file');
            if (!$file) throw new Exception('File is missing', 1);

            $import = Excel::toArray(new BrandProtectionPlatformImport(), $file->store('bpImports'));
            $import = array_values($import)[0];

            // Get the id for Site's channel
            $objChannel = Channel::where('slug', 'site')->firstOrFail();
            // Setting to all websites to not be considered
            Links::where('channel_id', $objChannel->id)->update(['consider' => 'false']);

            $highMediumLow = [
                'High',
                'Medium',
                'Low',
            ];
            $assetType = $params['assetType']; // Default = Website
            $assessments = json_decode($params['assessments'], true);
            $executedAt = $params['executed_at'];
            $len = count($import);
            for ($i = 0; $i < $len; $i++) {
                if ($i === 0) continue; // Skipping headers
                // if ($i > 1) break;

                // Getting worksheet column index for each param
                $domain = trim($import[$i][self::columnToIndex($params['domain'])]);
                $brand = trim($import[$i][self::columnToIndex($params['brand'])]);
                $assetTypeColumnValue = trim($import[$i][self::columnToIndex($params['assetTypeColumn'])]);
                $country = trim($import[$i][self::columnToIndex($params['country'])]);
                $consider = trim($import[$i][self::columnToIndex($params['consider'])]);
                $environment = trim($import[$i][self::columnToIndex($params['environment'])]);
                $status = trim($import[$i][self::columnToIndex($params['status'])]);
                $scope = trim($import[$i][self::columnToIndex($params['scope'])]);
                $businessArea = trim($import[$i][self::columnToIndex($params['businessArea'])]);

                if (strtolower($assetTypeColumnValue) !== strtolower($assetType)) continue; // Next iteration if is not website

                $data = Links::where('link', $domain)->first();
                if (!is_null($data)) {
                    if (empty($brand)) {
                        $brand = $data->brand->name;
                    }
                    if (empty($country)) {
                        $country = $data->country->name;
                    }
                }

                $objCountry = Country::whereRaw('lower(name) = ? ', strtolower($country))->first();
                if (is_null($objCountry)) {
                    array_push($errors['country'], [
                        'domain' => $domain,
                        'country' => $country,
                        'brand' => $brand,
                    ]);
                    continue;
                }

                $objBrand = Brand::whereRaw('lower(name) = ? ', strtolower($brand))->first();
                if (is_null($objBrand)) {
                    $objBrand = new Brand();
                    $objBrand->name = $brand;
                    $objBrand->slug = app(BrandController::class)->generateSlug($brand);
                    $objBrand->save();
                }

                $domainLinks = Links::where('link', $domain)->get();
                if (count($domainLinks) === 0) {
                    $newLink = new Links();
                    $newLink->link = $domain;
                    $newLink->brand_id = $objBrand->id;
                    $newLink->channel_id = $objChannel->id;
                    $newLink->country_id = $objCountry->id;
                    $newLink->http_status = $status;
                    $newLink->business_area = $businessArea;
                    $newLink->environment = $environment;
                    $newLink->consider = (trim(strtolower($consider)) === 'yes');
                    $newLink->scope = $scope;
                    $newLink->save();

                    $ownership = new Ownership();
                    $ownership->link_id = $newLink->id;
                    $ownership->save();
                } else {
                    foreach ($domainLinks as $value) {
                        $value->update(array(
                            'brand_id' => $objBrand->id,
                            'channel_id' => $objChannel->id,
                            'country_id' => $objCountry->id,
                            'http_status' => $status,
                            'business_area' => $businessArea,
                            'environment' => $environment,
                            'consider' => (trim(strtolower($consider)) === 'yes'),
                        ));
                    }
                }

                if (!$assessments) continue;
                foreach ($assessments as $key => $value) {
                    $criteriaId = $value['assessment']['id'];
                    $columns = explode(';', $value['assessmentColumn']);

                    $valueCriteria = '';
                    foreach ($columns as $keyColumn => $valueC) {
                        $valueColumn = trim($import[$i][self::columnToIndex($valueC)]);

                        // This is a High/Medium/Low case, calculate the correct level - Normally Vulnerability level
                        if (count($columns) > 1) {
                            if ((is_numeric($valueColumn)) && (intval($valueColumn) > 0)) {
                                $valueCriteria = $highMediumLow[$keyColumn];
                                break;
                            } else {
                                $valueCriteria = $highMediumLow[count($highMediumLow) - 1];
                            }
                        } else {
                            $valueCriteria = $valueColumn;
                        }
                    }

                    $paramsToLc = json_encode(array(
                        'criteriaId' => $criteriaId,
                        'response' => $valueCriteria,
                        'link' => $domain,
                        'executed_at' => $executedAt,
                        'official' => true,
                    ));
                    app(LinkCriteriaController::class)->store($paramsToLc)->getOriginalContent();
                }
            }

            $return = [
                'status' => 'success',
                'data' => array(
                    'success' => true
                ),
                'errors' => $errors,
            ];
            $statusCode = 200;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to get!'),
                'detail' => $e->getMessage(),
                'line' => $e->getLine(),
                'errors' => $errors,
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Save Spreadsheet
     *
     * @param Request $request
     *
     * @return JsonResponse
     */
    public function saveSpreadsheet(Request $request): JsonResponse
    {
        try {
            $file = $request->file('file');
            if (!$file) throw new Exception('File is missing', 1);
            $headers = [];
            $address = $file->store('bpImports');

            $import = Excel::toArray(new BrandProtectionPlatformImport(), $address);
            $import = array_values($import)[0];
            $headers = $import[0];
            $return = [
                'status' => 'success',
                'data' => array(
                    'file' => $address,
                    'headers' => $headers
                ),
            ];
            $statusCode = 200;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to get!'),
                'detail' => $e->getMessage(),
                'line' => $e->getLine(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }
}
