<?php

namespace App\Http\Controllers;

use App\Exports\ReportExport;
use App\Http\Controllers\Helpers\HelpersController;
use App\Models\NpsFeedbacks;
use DateTime;
use Exception;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Maatwebsite\Excel\Facades\Excel;
use Symfony\Component\HttpFoundation\BinaryFileResponse;

/**
 * NpsFeedbackController Controller
 *
 * @author Tércio Lacerda <Tercio.lacerda-ext@ab-inbev.com>
 * @package App\Http\Controllers
 */
class NpsFeedbackController extends Controller
{
	/**
	 * Map of request parameters to respective table.column in db
	 */
	protected $bdMapColumns = [
		'dateRange' => 'nps_feedbacks.created_at',
		'users' => 'users.email',
		'feedbackRating' => 'nps_feedbacks.rating',
	];

	/**
	 * Export feedbacks
	 *
	 * @param Request $request
	 *
	 * @return BinaryFileResponse|JsonResponse
	 */
	public function exportFeedbacks(Request $request): BinaryFileResponse|JsonResponse
	{
		try {
			$query = NpsFeedbacks::query()->join('users', 'nps_feedbacks.user_id', '=', 'users.id');

			HelpersController::buildRequestWhere($query, $request->all(), $this->bdMapColumns);

			$query = $query->orderBy('nps_feedbacks.created_at', 'desc')
				->get([
					'users.email as email',
					'nps_feedbacks.rating as rating',
					'nps_feedbacks.feedback_message as feedbackMessage',
					'nps_feedbacks.created_at as createdAt',
				])->toArray();

			$toExport = array();
			$headers = array(
				'Email',
				'Rating',
				'Feedback Message',
				'Date',
			);
			array_push($toExport, $headers);

			foreach ($query as $key => $value) {
				array_push($toExport, $value);
			}

			$report = new ReportExport($toExport);
			$date = new DateTime();
			$filename = 'export-feedbacks-' . $date->format('Y-m-d') . '.xlsx';
			$return = Excel::download($report, $filename);
		} catch (Exception $e) {
			$return = response()->json([
				'status' => 'error',
				'message' => __('Failed to get!'),
				'detail' => $e->getMessage(),
				'line' => $e->getLine(),
			]);
		}
		return $return;
	}

	/**
	 * Get a paginated list of feedbacks
	 *
	 * @param Request $request
	 *
	 * @return JsonResponse
	 */
	public function getFeedbacks(Request $request): JsonResponse
	{
		try {
			$query = NpsFeedbacks::query()->join('users', 'nps_feedbacks.user_id', '=', 'users.id');

			HelpersController::buildRequestWhere($query, $request->all(), $this->bdMapColumns);

			$query = $query->orderBy('nps_feedbacks.created_at', 'desc')
				->paginate($request->per_page, [
					'users.email as email',
					'nps_feedbacks.rating as rating',
					'nps_feedbacks.feedback_message as feedbackMessage',
					'nps_feedbacks.created_at as createdAt',
				]);

			$resume = NpsFeedbacks::selectRaw('count(*) as total, rating')->groupBy('rating')->get();

			$total = NpsFeedbacks::count();

			$statusCode = 200;
			$return = [
				'status' => 'success',
				'data' => array(
					'items' => $query,
					'resume' => $resume,
					'totalFeedbacks' => $total,
				),
			];
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => __('Failed to get!'),
				'detail' => $e->getMessage(),
				'line' => $e->getLine(),
			];
			$statusCode = 500;
		}
		return response()->json($return, $statusCode);
	}

	/**
	 * Get the date of last feedback by a user
	 *
	 * @param Request $request
	 *
	 * @return JsonResponse
	 */
	public function getLastFeedbackByUser(Request $request): JsonResponse
	{
		try {
			$lastFeedbackDate = NpsFeedbacks::where('user_id', $request->userReq->userId)->orderBy('created_at', 'desc')->first(['created_at']);

			$statusCode = 200;
			$return = [
				'status' => 'success',
				'data' => $lastFeedbackDate,
			];
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => __('Failed to get!'),
				'detail' => $e->getMessage(),
				'line' => $e->getLine(),
			];
			$statusCode = 500;
		}
		return response()->json($return, $statusCode);
	}

	/**
	 * Register a NPS Feedback by a user
	 *
	 * @param Request $request
	 *
	 * @return JsonResponse
	 */
	public function userFeedback(Request $request): JsonResponse
	{
		try {
			$params = $request->all();

			if ((empty($params['rating']) || strlen($params['rating']) === 0)) {
				throw new Exception('Required params are missing.', 1);
			}

			$newNpsFeedback = new NpsFeedbacks();
			$newNpsFeedback->user_id = $request->userReq->userId;
			$newNpsFeedback->rating = $params['rating'];
			$newNpsFeedback->feedback_message = (empty($params['feedbackMessage']) || strlen($params['feedbackMessage']) === 0) ? '' : $params['feedbackMessage'];
			$newNpsFeedback->save();

			$statusCode = 200;
			$return = [
				'status' => 'success',
				'message' => __('Created successfully'),
			];
		} catch (Exception $e) {
			$return = [
				'status' => 'error',
				'message' => __('Failed to get!'),
				'detail' => $e->getMessage(),
				'line' => $e->getLine(),
			];
			$statusCode = 500;
		}
		return response()->json($return, $statusCode);
	}
}
