<?php

namespace App\Http\Controllers;

use App\Models\Requests;
use Exception;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;

/**
 * Requests Controller
 *
 * @author Lucas Buzzo <lucas.buzzo@ab-inbev.com>
 * @package App\Http\Controllers
 */

class RequestsController extends Controller
{
    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     *
     * @return JsonResponse
     */
    public function destroy(int $id): JsonResponse
    {
        try {
            Requests::destroy($id);
            $return = [
                'status' => 'success',
                'message' => __('Deleted successfully!'),
            ];
            $statusCode = 201;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to delete!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Display a listing of the resource.
     *
     * @return JsonResponse
     */
    public function index(): JsonResponse
    {
        try {
            $requests = Requests::all();

            $return = [
                'status' => 'success',
                'data' => $requests,
            ];
            $statusCode = 200;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to get!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Display a paginated listing of the resource.
     *
     * @param ?Request $request
     *
     * @return JsonResponse
     */
    public function index_paginated(Request $request = null): JsonResponse
    {
        try {
            $perPage = (!is_null($request)) ? $request->per_page : 20;
            $requets = Requests::paginate($perPage);

            $return = [
                'status' => 'success',
                'data' => $requets,
            ];
            $statusCode = 200;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to get!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Show the requets for a given id.
     *
     * @param int $id
     *
     * @return JsonResponse
     */
    public function show(int $id): JsonResponse
    {
        try {
            $request = Requests::find($id);

            $return = [
                'status' => 'success',
                'data' => $request,
            ];

            $statusCode = 200;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to get!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param string $params
     *
     * @return JsonResponse
     */
    public function store(string $params): JsonResponse
    {
        $params = json_decode($params, true);

        try {
            $request = new Requests();

            $request->request_type = $params['request_type'];

            if (isset($params['user_id'])) {
                $request->user_id = $params['user_id'];
            }

            if (isset($params['approved'])) {
                $request->approved = $params['approved'];
            } else {
                $request->approved = false;
            }

            $request->save();

            $return = [
                'status' => 'success',
                'message' => __('Created successfully'),
            ];
            $statusCode = 201;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to Create!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param int $id
     * @param string $params
     *
     * @return JsonResponse
     */
    public function update(int $id, string $params): JsonResponse
    {
        $params = json_decode($params, true);

        try {
            $request = Requests::find($id);

            if (isset($params['request_type'])) {
                $request->request_type = $params['request_type'];
            }

            if (isset($params['user_id'])) {
                $request->user_id = $params['user_id'];
            }

            if (isset($params['approved'])) {
                $request->approved = $params['approved'];
            } else {
                $request->approved = false;
            }

            $request->save();

            $return = [
                'status' => 'success',
                'message' => __('Updated successfully'),
            ];
            $statusCode = 200;
        } catch (Exception $e) {
            $return = [
                'status' => 'error',
                'message' => __('Failed to Update!'),
                'details' => $e->getMessage(),
            ];
            $statusCode = 500;
        }
        return response()->json($return, $statusCode);
    }
}
