# Crawling Process

Our crawling process has multiple steps:

1. Get the information from the database about the links that will be evaluated
2. Define which crawler will be executed (for each one the process is different):
   - Website
   - Facebook
   - Instagram
   - YouTube
   - Twitter
3. Using Puppeteer\* create a Chromium instance, with additional configuration to avoid bot detection
4. Validated if the website is online (not an offline website or redirecting to another domain)
5. Check if will be necessary additional steps, such:
   - Age gate bypass, for cases the age gate is on another page and we cannot access the content
   - Cookie banner interaction, programmatically accepting all cookies
   - Based on frames inside the page, generate a list with all iframes and retrieve content from them
   - Generate a list of all requests that the website made (internal or external)
6. Once we've all the source code/text from the page (including the iframes content), we run the evaluations
7. Before the evaluations, normalize all the content we've got from our crawler
8. For IARD
   1. Retrieve the keywords for each criterion of IARD for the location of the asset
   2. Search for those keywords in the normalized text
   3. Save in the database the result for this asset about this specific criteria
9. For our data capture validation (Google Analytics implementation):
   1. Based on requests that the website made, we search for a "pageview" event sent to Google Analytics
   2. Retrieve the account id that received this hit
      1. Validate if we've access to this account
      2. Check if the account name is following our taxonomy pattern
   3. Save in the database the result for this website about this specific criteria
10. After we've all the information, we export a report to do the manual validations
11. Posterior to the manual validation, we import the information and overwrite the automation data
12. Refresh our materialized view
13. Clear the cache

## Why use Puppeteer?

Two reasons that made us choose Puppeteer instead of just performing a request, in a more performant way, like curl:

- Some interactions needed to be made with the website, like age gate bypass;
- Have the same experience that users have, including the problem with Client Side Rendered applications

### Age gate bypass

After all the initial validations, to ensure the website is online and is not a redirect, we have the process to bypass the age gate. This is a required step because we’ve some websites where the content will only be available after the age gate.

Let’s take, for example, the domain: https://www.bohemia.com.br/. After we open this website, we’re redirected to https://www.bohemia.com.br/age (without cookie banner). So, to correctly evaluate this website, our automation needs to be able to bypass the age gate.

The easier way to do that, and our automation is not dependent on the information available in the frontend of the website (interacting with an element with a specific class, filling in some information, and clicking on a button) is to deeply understand how the website works after the user informs the age.

In this case, we can see that there is a cookie being dropped to store the information related to the age of the user if the user is above the legal drinking age:

<img :src="$withBase('/images/agegate-brahma.png#center')" alt="Age gate bypass">

The cookie name is: “bohemia_age_gate” and the value is “verified”. We’ve stored in a database a long list with this type of instruction so our automation knows how to bypass the age gate, the biggest problem here is we don’t have a pattern, Bohemia uses a cookie called “bohemia_age_gate”, Skol.com.br uses a cookie called “isUnderage” – so, in the first moment, this information needs to be discovered manually to store in the database, after that all the access from our automation will be able to bypass the age gate.

Currently, this project supports 5 different ways to bypass the age gate:

1. Cookie: defining one (or more) cookies with a specific value;
2. Local storage: defining one (or more) variables in local storage with a specific value;
3. Session storage: defining one (or more) variables in session storage with a specific value;
4. Redirect: cases where the user is redirected to another page that set the information that the user is above LDA (Legal Drinking Age);
5. Steps: steps that the robot needs to do to bypass the age gate (click on elements, fill in specific information, submit forms, etc).

### Client Side Rendered applications

To illustrate the problem, we can use DGP application - that is a Single Page Application, created with VueJs and rendered on the client side.

The homepage of the DGP application looks like that:

<img :src="$withBase('/images/abi-dgp-home.png#center')" alt="ABI DGP Homepage">

Is possible to see that there are some text entries, like the sidebar, the titles of the page and charts, etc. Using curl to retrieve the source code of the application, the result will be:

<img :src="$withBase('/images/curl-abi-dgp-home.png#center')" alt="curl sourcecode ABI DGP homepage">

Using this approach, we'll retrieve exact the source code from the application. However, solutions like React/Vue/Svelte are gaining market share and a lot of modern applications are made using client-side render solutions. To have the same behavior that a user will have is necessary to download, parse and execute the JavaScript code that is in the application.

Puppeteer allows us to interact programmatically with a Chromium instance – this way, we can have the same experience that a user would have to open our website, not just the communication between two servers.

Another advantage of this approach is that we can run this Chromium instance in a headless mode, so we can run all our code in any server (or serverless) without having to pay more to have a graphical interface.

## Ensuring an asset is working

This process is made in steps, to ensure that we've covered all the scenarios and not just validated the response code. We had a few websites that returned the HTTP code "OK" (200), but after a few seconds, the application is redirected using a JavaScript code.

### Site online validation

To validate the status of one website, and to know if the website is online or offline, we have a few steps:

<img :src="$withBase('/images/site-status-validation.png#center')" alt="Site Status Validation">

First, our Chromium instance will try to go to the link. And then, we can check which will be the response from the Chromium. The method **goto** will return an **HTTPReponse** or **null** – if the return is null, the website is offline.

Additionally, we’ll check the HTTP response code for the website and any status bigger than 400 will be handled like an offline website. In this case, we intentionally decide to allow redirect (3xx) and will just check for error status, as you can see below:

<img :src="$withBase('/images/http-status-code.png#center')" alt="HTTP Status Code">

### Redirect validation

We mentioned in the Site status validation that we intentionally decide to not check for redirects, and the reason is that it would not be the more precise way to check for redirects, in our case. In the general context, is considered a redirect when the user tries to open the website in a non-HTTPS and the website forces the user to see the HTTPS version – the HTTP code for this will be 30x, and for us, this is not a redirect.

Our validation of redirect consists of the below:

<img :src="$withBase('/images/redirect-validation.png#center')" alt="HTTP Status CodeRedirect validation">

First, we’ll turn the link (this is the initial link, the one we are evaluating) into a URL, this makes it easier to manipulate the information when compared to the other possible approach of manipulating the text. In the same way, we’ll check the current link (linkFinal, this is the final URL we got after opening the website) into a URL.

To ensure we’re comparing the same information, we’ll just check the hostname – you can see a brief explanation about which part of the URL is the hostname in the below image:

<img :src="$withBase('/images/url-structure.png#center')" alt="URL structure">

Comparing just the hostname, we ensure that if for some reason, we have an additional path (for example the age gate is in another path, not just a modal opening in front of the content) or query in the URL, we’ll compare the same information. The last step is to remove the “www.” From the URL to ensure we have the same information.

After all these manipulations in the link, if the information is different, the website is considered a redirect.
